package accu.browser.functional;


public class ListBoxModel {
	public final Integer size;
	public final Window window;
	
	public ListBoxModel(Integer size, Window window) {
		this.size = size;
		this.window = window;
	}
	
	public ListBoxModel withWindow(Window aWindow) {
	  return new ListBoxModel(size, aWindow);
	}
	
	public static ListBoxModel reflect(ListBoxModel model) {
		return model.withWindow(Window.reflectWindow(model.size, model.window));
	}
	
	public static ListBoxModel selectTop(ListBoxModel model) {
		return new ListBoxModel(model.size, 
				new Window(model.window.top, model.window.bottom, model.window.top));
	}
	
	public static ListBoxModel arrowUp(ListBoxModel model) {
		if (!Window.selectionAtEdge(model.window)) 
			return model.withWindow(Window.shiftSelection(model.window));
		if (Window.windowAtEdge(model.window))
			return model;
		return model.withWindow(Window.shiftWindow(model.window));
	}
	
	public static ListBoxModel arrowDown(ListBoxModel model) {
		return reflect(arrowUp(reflect(model)));
	}
	
	interface ModelFunction {
		ListBoxModel apply(ListBoxModel model);
	}
	
	public static ListBoxModel iterate(ModelFunction function, ListBoxModel model, Integer count) {
		ListBoxModel iterated = model;
		for(int n = 0; n < count; n++) {
			iterated = function.apply(iterated);
		}
		return iterated;
	}
	
	public static ListBoxModel pageWith(ModelFunction step, ListBoxModel model) {
		return selectTop(iterate(step, selectTop(model), Window.windowSize(model.window)));	
	}
	
	public static ListBoxModel pageDown(ListBoxModel model) {
		return pageWith(arrowDownFunction(), model);
	}
	
	public static ListBoxModel pageUp(ListBoxModel model) {
		return pageWith(arrowUpFunction(), model);
	}
	
	private static ModelFunction arrowUpFunction() {
	  return new ModelFunction() {
      public ListBoxModel apply(ListBoxModel aModel) { return arrowUp(aModel); }
    };
	}
  private static ModelFunction arrowDownFunction() {
    return new ModelFunction() {
      public ListBoxModel apply(ListBoxModel model) { return arrowDown(model); }
    };
  }
}
